                            % ANESTHESIA MODEL FOR BENCHMARK PATIENT SIMULATOR
                                    % CLARA IONESCU AND DANA COPOT
                                % GHENT UNIVERSITY, BELGIUM, JANUARY 2021

%when this program is used please cite the following: 
% An Open Source Patient Simulator for Design and Evaluation of Computer  Based Multiple Drug Dosing Control for Anesthetic and Hemodynamic Variables
% C. Ionescu, M. Neckebroek, Mi. Ghita and D. Copot
% IEEE Access; 2021; doi 10.1109/ACCESS.2021.3049880  
                                
          

%% Initialisation of anesthesia models
%Anesthesia_model consists of 3 parts: Hypnosis, Analgesia,neuromuscular blockade

%% Hypnosis
%Input is Propofol, output is BIS (bispectral index)
%Transfer function from Propofol input to effect site concentration
Ap = [-(k10p + k12p + k13p) k21p k31p 0; k12p -k21p 0 0; k13p 0 -k31p 0; k1ep 0 0 -ke0p];
Bp = [1;0;0;0];
Cp = [0 0 0 1];
Dp = 0;
propSS = ss(Ap,Bp,Cp,Dp);

%% Analgesia
%Transfer function from Remifentanil input to effect concentration
Ar = [-(k10r + k12r + k13r) k21r k31r 0; k12r -k21r 0 0; k13r 0 -k31r 0; k1er 0 0 -ke0r];
Br = [1;0;0;0];
Cr = [0 0 0 1];
Dr = 0;
remiSS = ss(Ar,Br,Cr,Dr);

%Model from Remifentanil to level RASS sedation score
sysRASS = tf(-2, [1 2]); %Generate transfer function of continuous model
[numRass,denRass]=tfdata(sysRASS,'v'); %Get coefficients of transfer function of continuous model

% Note: this model also includes a gain dependent on the current effect site
% concentration of Remifentanil. This is implemented directly in the
% simulink-model

%% Neuromuscular blockade (NMB)
%NMB model
sysNMB=zpk([],[-k1a*alphaNMB, -k2a*alphaNMB, -k3a*alphaNMB],k1a*k2a*k3a*alphaNMB^3); %Generate transfer function of nominal model
[numN,denN]=tfdata(sysNMB,'v'); %Get coefficients of transfer function of nominal model
[Anmb,Bnmb,Cnmb,Dnmb] = tf2ss(numN,denN); %Convert transfer function to state space
nmbSS = ss(Anmb,Bnmb,Cnmb,Dnmb); %Generate state space model


